<?php

/*
CLASSE COPIADA COM MODIFICAÇÕES DA DEPENDENCIA 'TEKNISA/LIBRARIES' VERSÃO: '3.0.24'. NAMESPACE ORIGINAL 'Teknisa\Libs\Util'
*/

namespace EMS\PortalFuncionario\Auth;

use Zeedhi\Framework\Events\PreDispatch\Listener;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\Session\Session;

use EMS\PortalFuncionario\Util\Messages;
use EMS\PortalFuncionario\Auth\Exception\AccessException;

class RequestListenerOAuth extends Listener {

    public function __construct(Kernel $kernel, OAuthCustom $oauth, Session $sessionManager) {
        $this->httpKernel       = $kernel;
        $this->oauth            = $oauth;
        $this->sessionManager   = $sessionManager;
        $this->publicRoutes     = $this->readPublicRoutes();
    }

    const           AUTHORIZATION_REQUEST_HEADER = 'OAuth-Token';
    private const   DEFAULT_TOKEN_VALUE = null;

    const           KEEP_CONNECTED_POSITIVE = 'Yes';
    const           PUBLIC_ROUTES_PATH = '/../../../config/routes/publicRoutes.json';

    public function preDispatch(DTO\Request $request) {
        $headers = $this->httpKernel->getHttpRequest()->getHeaders();
        $apacheHeaders = apache_request_headers();

        $route = $request->getRoutePath();
        $isRoutePublic = $this->isRoutePublic($route);

        if ($isRoutePublic) {
            return;
        }
        
        $token = $this->getRequestToken();

        if (!is_null($token)) {
            $service = $this->checkAccessAndSetSession($token);
        } else {
            throw new AccessException();
        }
    }

    public function checkAccessAndSetSession($token) {
        $secretId = null;
        $keepConn = true;
        $access   = $this->oauth->checkAccess($token, $secretId, array(), $keepConn);

        if (!empty($access)) {
            if($this->sessionManager->getId() != $access['sessionId']) {
                $this->sessionManager->start();
                $this->sessionManager->destroy();
                $this->sessionManager->setId($access['sessionId']);
            }
            $this->sessionManager->start();

            if(!$this->sessionManager->has("NRORG")) {
                $this->sessionManager->set('NRORG', $access['service']->getNrOrg());
                $this->sessionManager->set('CDOPERADOR', $access['service']->getCdOperator());
            }

            if(!$this->sessionManager->has("ORGANIZATION_ID")) {
                $this->sessionManager->set('ORGANIZATION_ID', $access['service']->getNrOrg());
                $this->sessionManager->set('USER_ID', $access['service']->getCdOperator());
            }
        }
    }

    private function readPublicRoutes() {
        $publicRoutesPath = $this->getPublicRoutesPath();
        $urls = file_get_contents($publicRoutesPath);
        return json_decode($urls, true);
    }

    private function getPublicRoutesPath() {
        return __DIR__ . self::PUBLIC_ROUTES_PATH;
    }

    private function isRoutePublic($route) {
        return in_array($route, $this->publicRoutes);
    }

    // Busca pelo nome do 'Header' do token, com o case original ou em lowercase
    private function getRequestToken() {
        return $this->httpKernel->getHttpRequest()->getHeaders()->get(self::AUTHORIZATION_REQUEST_HEADER);
    }

}