<?php
namespace EMS\PortalFuncionario\Geral\Repository;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;

class ApuracaoRepository {

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    public function getPeriodoApuracao(
        string $nrvinculom,
        string $nrorg
    ) : array {
        try {
            $sql = <<<SQL
                SELECT * FROM 
                ( 
                    SELECT PPA.NRPERIODOAPURACAO,
                        TO_CHAR(PPA.DTINICIALAPURACAO, 'DD/MM/YYYY')||' - '||TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY')||
                        ' ('||PPA.IDPERIODOAPURACAO||')'  AS DSPERIODOAPURACAO,
                        PPA.NRORG,
                        TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY') AS DTFINALAPURACAO, 
                        TO_CHAR(PPA.DTINICIALAPURACAO, 'DD/MM/YYYY') AS DTINICIALAPURACAO, 
                        PPA.NRTIPOPERIODOAPURACAO, 
                        PPA.IDPERIODOAPURACAO
                    FROM PTO_PERIODOAPURACAO PPA, PTO_TIPOPERIODOAPURACAO TPA
                    JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = :P_NRVINCULOM AND VM.NRORG = :P_NRORG
                    WHERE PPA.NRORG                   = :P_NRORG
                        AND TPA.NRTIPOVINCULOM        = VM.NRTIPOVINCULOM
                        AND PPA.DTINICIALAPURACAO    <= LAST_DAY(SYSDATE)
                        AND PPA.NRTIPOPERIODOAPURACAO = TPA.NRTIPOPERIODOAPURACAO
                        AND PPA.NRORG                 = TPA.NRORG
                    ORDER BY PPA.DTINICIALAPURACAO DESC
                )
                WHERE ROWNUM <= 30
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"             => $nrorg,
                    "P_NRVINCULOM"        => $nrvinculom
                )
            );

            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getPeriodoApuracaoAtual(
        string $nrvinculom,
        string $nrorg
    ) : array {
        try {
            $sql = <<<SQL
                SELECT * FROM 
                ( 
                    SELECT PPA.NRPERIODOAPURACAO,
                        TO_CHAR(PPA.DTINICIALAPURACAO, 'DD/MM/YYYY')||' - '||TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY')||
                        ' ('||PPA.IDPERIODOAPURACAO||')'  AS DSPERIODOAPURACAO,
                        PPA.NRORG,
                        TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY') AS DTFINALAPURACAO, 
                        TO_CHAR(PPA.DTINICIALAPURACAO, 'DD/MM/YYYY') AS DTINICIALAPURACAO, 
                        PPA.NRTIPOPERIODOAPURACAO, 
                        PPA.IDPERIODOAPURACAO
                    FROM PTO_PERIODOAPURACAO PPA, PTO_TIPOPERIODOAPURACAO TPA
                    JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = :P_NRVINCULOM AND VM.NRORG = :P_NRORG
                    WHERE PPA.NRORG                   = :P_NRORG
                        AND TPA.NRTIPOVINCULOM        = VM.NRTIPOVINCULOM
                        AND PPA.DTINICIALAPURACAO     <= LAST_DAY(SYSDATE)
                        AND PPA.NRTIPOPERIODOAPURACAO = TPA.NRTIPOPERIODOAPURACAO
                        AND PPA.NRORG                 = TPA.NRORG
                    ORDER BY PPA.DTINICIALAPURACAO DESC
                )
                WHERE ROWNUM <= 1
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"             => $nrorg,
                    "P_NRVINCULOM"        => $nrvinculom
                )
            );

            $queryResult = $query->fetchAll();

            return empty($queryResult) ? array() : $queryResult[0];
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getPeriodo(
        string $nrorg, 
        string $nrperiodopuracao
    ) {
        $sql = <<<SQL
            SELECT
                TO_CHAR(PPA.DTINICIALAPURACAO, 'DD/MM/YYYY')||' - '||TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY')||
                ' ('||PPA.IDPERIODOAPURACAO||')'  AS DSPERIODOAPURACAO,
                PPA.NRORG,
                TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY') AS DTFINALAPURACAO, 
                TO_CHAR(PPA.DTINICIALAPURACAO, 'DD/MM/YYYY') AS DTINICIALAPURACAO, 
                PPA.NRTIPOPERIODOAPURACAO, 
                PPA.IDPERIODOAPURACAO
            FROM
                PTO_PERIODOAPURACAO PPA
            WHERE
                PPA.NRORG = :P_NRORG
                AND PPA.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
SQL;

        $params = array(
            'P_NRORG' => $nrorg,
            'P_NRPERIODOAPURACAO' => $nrperiodopuracao
        );
        
        $result = $this->entityManager->getConnection()->fetchAll($sql, $params);

        return empty($result) ? array() : $result[0];
    }

    public function getApuracaoPorData(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao,
        string $dtapuracaoponto,
        string $dtmescompetencia
    ) {

        /* retorna os resumos de situação de frequência, as marcações de entrada/saída, e ausências totalizadas  */
        $resumosSitufreq = $this->getResumosSitufreqPorData(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao,
            $dtapuracaoponto,
            $dtmescompetencia
        );

        /* verifica se existe ocorrencia de frequência no dia para compensação de banco de horas */
        $ocorrenciasCompensarBH = $this->verificaOcorreciaCompensacaoBH(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* verifica se existe ocorrencia de frequência no dia para autorização de hora extra e verifica o status da mesma */
        $ocorrenciasAutorizacaoEX = $this->verificaOcorrenciaAutorizacaoEX(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* verifica se existe ocorrencia de frequência no dia para identificação de intervalos */
        $ocorrenciasIntervalos = $this->verificaIdentificaIntervalo(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* verifica se existe ocorrencia de frequência no dia para abonar falta e verifica o status da mesma */
        $ocorrenciasAbonarFalta = $this->verificaAbonarFalta(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* verifica se existe ocorrencia de frequência no dia para compensar falta e verifica o status da mesma */
        $ocorrenciasCompensarFalta = $this->verificaCompensarFalta(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* verifica se existem ocorrencias pendentes para o vinculo no dia a ser apurado */
        $ocorrenciasPendentes = $this->verificaExisteOcorrenciaPendente(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* Verifica se existem resumos de situação de frequência de faltas diurnas ou noturnas para o dia*/
        $faltasDiurnasNoturnas = $this->verificaExisteResumoFaltaDiurnaNoturna(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao,
            $dtapuracaoponto
        );

        /* Verifica se existe ocorrências de feriado no calendário para a data */
        $ocorrenciasCalendario = $this->verificaOcorrenciasCalendario(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto,
            $dtmescompetencia
        );

        $apuracaoPonto = $this->factory(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao,
            $dtapuracaoponto,
            $dtmescompetencia,
            $resumosSitufreq,
            $ocorrenciasCompensarBH,
            $ocorrenciasAutorizacaoEX,
            $ocorrenciasIntervalos,
            $ocorrenciasAbonarFalta,
            $ocorrenciasCompensarFalta,
            $ocorrenciasPendentes,
            $faltasDiurnasNoturnas,
            $ocorrenciasCalendario
        );

        return $apuracaoPonto;

    }

    public function getResumosSitufreqPorData(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao,
        string $dtapuracaoponto,
        string $dtmescompetencia
    ) {
        $sql = <<<SQL
            SELECT
                APU_DIA.ENTRADA,
                APU_DIA.ALMOCO,
                APU_DIA.SAIDA,
                APU_DIA.AU_INJU,
                APU_DIA.AU_JUST,
                APU_DIA.AU_TOTA,
                APU_DIA.TRABALH,
                APU_DIA.DIF_TRA,
                APU_DIA.HE_AUTO,
                APU_DIA.HE_NAUT,
                APU_DIA.FL_ATRA,
                APU_DIA.FL_DIAS,
                APU_DIA.TRAB_NORM,
                APU_DIA.BH_DESC_DIA,
                APU_DIA.ID_PROGOCOR,
                APU_DIA.ID_DIAAPURADO,
                APU_DIA.ADICIONAL_NOTURNO,
                PO.IDEXIBEBANCOHRS,
                CASE
                    WHEN NVL(PO.IDEXIBEBANCOHRS, 'N') <> 'S' THEN
                        '-'
                    ELSE TO_CHAR(APU_DIA.BH_DIA_ANT)
                END AS BH_DIA_ANT,
                CASE
                    WHEN NVL(PO.IDEXIBEBANCOHRS, 'N') <> 'S' THEN
                        '-'
                    ELSE TO_CHAR(APU_DIA.BH_S_AC)
                END AS BH_S_AC,
                CASE
                    WHEN NVL(PO.IDEXIBEBANCOHRS, 'N') <> 'S' THEN
                        '-'
                    ELSE TO_CHAR(APU_DIA.BH_C_AC)
                END AS BH_C_AC
            FROM (
                SELECT
                    *
                FROM TABLE (PONTODEMONST_DADOS(TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY'), :P_NRORG, TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'), :P_NRPERIODOAPURACAO, :P_NRVINCULOM))
            ) APU_DIA
            LEFT JOIN GPE_PARAMORG PO ON PO.NRORG = :P_NRORG
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                "P_DTMESCOMPETENCIA"  => $dtmescompetencia,
                "P_NRORG"             => $nrorg,
                "P_DTAPURACAOPONTO"   => $dtapuracaoponto,
                "P_NRPERIODOAPURACAO" => $nrperiodoapuracao,
                "P_NRVINCULOM"        => $nrvinculom
            )
        );

        $apuracaoPonto = $query->fetchAll();
        return $apuracaoPonto[0];
    }

    private function verificaOcorreciaCompensacaoBH(
        $nrvinculom,
        $nrorg,
        $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
               COUNT(*) AS COUNT_COMPENSACAO_BH
              FROM PTO_PROGOCORRENCIA
             WHERE NRORG                 = :P_NRORG
               AND NRVINCULOM            = :P_NRVINCULOM
               AND IDATIVO               = 'S'
               AND NRTIPOPROGOCORRENCIA IN (2, 10, 18, 19)
               AND DTINICIOPROGOCOR      = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
               AND EXISTS (SELECT 1
                             FROM GPE_MARCACAOPONTO
                            WHERE NRORG      = PTO_PROGOCORRENCIA.NRORG
                              AND NRVINCULOM = PTO_PROGOCORRENCIA.NRVINCULOM
                              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrencias = $query->fetchAll();
        return $ocorrencias[0];
    }

    private function verificaOcorrenciaAutorizacaoEX(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
               COUNT(*) AS COUNT_AUTORIZACAO_EX,
               IDPROGOCORCONFIRMADA
              FROM PTO_PROGOCORRENCIA
             WHERE NRORG                = :P_NRORG
               AND NRVINCULOM           = :P_NRVINCULOM
               AND NRTIPOPROGOCORRENCIA = 1
               AND IDATIVO              = 'S'
               AND (DTINICIOPROGOCOR     = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
                   OR DTFIMPROGOCOR IS NULL)
             GROUP BY IDPROGOCORCONFIRMADA
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrencias = $query->fetchAll();
        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }

    private function verificaIdentificaIntervalo(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT COUNT(*) AS COUNT_IDENTIFICA_INTERVALO
              FROM PTO_PROGOCORRENCIA PROG
              JOIN GPE_MARCACAOPONTO MARC
                ON MARC.NRMARCACAOPONTO = PROG.NRMARCACAOPONTO
               AND MARC.NRVINCULOM      = PROG.NRVINCULOM
               AND MARC.NRORG           = PROG.NRORG
             WHERE PROG.NRORG                 = :P_NRORG
               AND PROG.NRVINCULOM            = :P_NRVINCULOM
               AND PROG.IDATIVO               = 'S'
               AND PROG.NRTIPOPROGOCORRENCIA IN (20, 26)
               AND PROG.DTINICIOPROGOCOR      = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
               AND PROG.NRMARCACAOPONTO IS NOT NULL
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrencias = $query->fetchAll();
        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }

    private function verificaAbonarFalta(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
                COUNT(*) AS COUNT_ABONAR_FALTA,
                IDPROGOCORCONFIRMADA,
                IDTIPODIACALENDARIO,
                NRPROGOCORRENCIA,
                DSOBSERVACAO,
                DSOBSERVACAORECUSA
              FROM PTO_PROGOCORRENCIA PROG
             WHERE PROG.NRORG                = :P_NRORG
               AND PROG.NRVINCULOM           = :P_NRVINCULOM
               AND PROG.NRTIPOPROGOCORRENCIA = 5
               AND IDATIVO                   = 'S'
               AND TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY') BETWEEN PROG.DTINICIOPROGOCOR AND PROG.DTFIMPROGOCOR
               /*AND PROG.DTINICIOPROGOCOR     = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
               AND PROG.DTFIMPROGOCOR        = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')*/
               GROUP BY IDPROGOCORCONFIRMADA, IDTIPODIACALENDARIO, NRPROGOCORRENCIA, DSOBSERVACAO, DSOBSERVACAORECUSA
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrencias = $query->fetchAll();
        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }

    private function verificaCompensarFalta(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
                COUNT(*) AS COUNT_COMPENSACAO_FALTA,
                IDPROGOCORCONFIRMADA,
                NRPROGOCORRENCIA,
                DSOBSERVACAO,
                DSOBSERVACAORECUSA
              FROM PTO_PROGOCORRENCIA PROG
             WHERE PROG.NRORG                = :P_NRORG
               AND PROG.NRVINCULOM           = :P_NRVINCULOM
               AND NRTIPOPROGOCORRENCIA IN (2, 10, 18, 19)
               AND IDATIVO              = 'S'
               AND TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY') BETWEEN PROG.DTINICIOPROGOCOR AND PROG.DTFIMPROGOCOR
               AND NOT EXISTS (SELECT 1
                                 FROM GPE_MARCACAOPONTO
                                WHERE NRORG      = PROG.NRORG
                                  AND NRVINCULOM = PROG.NRVINCULOM
                                  AND NRCOLETOR  != 21
                                  AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
               GROUP BY IDPROGOCORCONFIRMADA, NRPROGOCORRENCIA, DSOBSERVACAO, DSOBSERVACAORECUSA
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrencias = $query->fetchAll();
        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }

    private function factory(
        $nrvinculom,
        $nrorg,
        $nrperiodoapuracao,
        $dtapuracaoponto,
        $dtmescompetencia,
        $resumosSitufreq,
        $ocorrenciasCompensarBH,
        $ocorrenciasAutorizacaoEX,
        $ocorrenciasIntervalos,
        $ocorrenciasAbonarFalta,
        $ocorrenciasCompensarFalta,
        $ocorrenciasPendentes,
        $faltasDiurnasNoturnas,
        $ocorrenciasCalendario
    ) {

        $apuracaoPonto = array();
        $apuracaoPonto["DTAPURACAOSITUFREQ"] = $dtapuracaoponto;

        if ($ocorrenciasCalendario['COUNT_OCORRENCIAS_FERIADO'] > 0) {
            $apuracaoPonto["OCORRENCIA_FERIADO"] = true;
        } else {
            $apuracaoPonto["OCORRENCIA_FERIADO"] = false;
        }

        if ($ocorrenciasPendentes['COUNT_OCORRENCIAS_PENDENTES'] > 0) {
            $apuracaoPonto["OCORRENCIAS_PENDENTES"] = true;
        } else {
            $apuracaoPonto["OCORRENCIAS_PENDENTES"] = false;
        }

        if ($faltasDiurnasNoturnas['COUNT_FALTAS'] > 0) {
            $apuracaoPonto["FALTA_IDENTIFICADA"] = true;
        } else {
            $apuracaoPonto["FALTA_IDENTIFICADA"] = false;
        }

        if ($ocorrenciasCompensarBH['COUNT_COMPENSACAO_BH'] > 0) {
            $apuracaoPonto["HORASDECONTADAS"]          = "00:00";                         /*diferença de horas a serem descontadas*/
            $apuracaoPonto["HORASBANCODIA"]            = $resumosSitufreq['BH_DESC_DIA']; /*diferença de horas a serem compensadas*/
            $apuracaoPonto["SOLICITOU_COMPENSACAO_BH"] = true;                            /*identificador de solicitação de hora extra*/
        } else {
            $apuracaoPonto["HORASDECONTADAS"]          = $resumosSitufreq['BH_DESC_DIA'];
            $apuracaoPonto["HORASBANCODIA"]            = "00:00";
            $apuracaoPonto["SOLICITOU_COMPENSACAO_BH"] = false;
        }

        if ($ocorrenciasAutorizacaoEX['COUNT_AUTORIZACAO_EX'] > 0) {
            $apuracaoPonto["HORASEXTRAS"]              = "00:00";                     /*horas extras não autorizadas*/
            $apuracaoPonto["HORASEXTRASAUTORIZADA"]    = $resumosSitufreq['HE_AUTO']; /*horas extras autorizadas*/

            if (floatval(str_replace(',', '.', $resumosSitufreq['HE_AUTO'])) > 0)
                $apuracaoPonto["SOLICITOU_HORAEXTRA"]      = true;                     /*identificador de solicitação de hora extra*/
            else
                $apuracaoPonto["SOLICITOU_HORAEXTRA"]      = false;                    /*identificador de solicitação de hora extra*/

            $apuracaoPonto["ID_SITUACAO_HE"]           = $ocorrenciasAutorizacaoEX['IDPROGOCORCONFIRMADA'];
        } else {
            $apuracaoPonto["HORASEXTRAS"]              = $resumosSitufreq['HE_NAUT'];
            $apuracaoPonto["HORASEXTRASAUTORIZADA"]    = "00:00";
            $apuracaoPonto["SOLICITOU_HORAEXTRA"]      = false;
            $apuracaoPonto["ID_SITUACAO_HE"]           = $ocorrenciasAutorizacaoEX['IDPROGOCORCONFIRMADA'];
        }

        if ($ocorrenciasIntervalos["COUNT_IDENTIFICA_INTERVALO"] > 0) {
            $apuracaoPonto["FALTA_MARCACAO_ALMOCO"] = false; /*identificador de identificação de intervalo*/
        } else {
            $apuracaoPonto["FALTA_MARCACAO_ALMOCO"] = true;
        }

        if ($ocorrenciasAbonarFalta["COUNT_ABONAR_FALTA"] > 0) {
            $apuracaoPonto["SOLICITOU_ABONAR_FALTA"]   = true;
            $apuracaoPonto["ID_SITUACAO_ABONARFALTA"]  = $ocorrenciasAbonarFalta['IDPROGOCORCONFIRMADA'];
            $apuracaoPonto["IDTIPODIACALENDARIO_OCOR"] = $ocorrenciasAbonarFalta['IDTIPODIACALENDARIO'];
            $apuracaoPonto["NRPROGOCORRENCIA_OCOR"]    = $ocorrenciasAbonarFalta['NRPROGOCORRENCIA'];
            $apuracaoPonto["DSOBSERVACAO_OCOR"]        = $ocorrenciasAbonarFalta['DSOBSERVACAO'];
            $apuracaoPonto["DSOBSERVACAORECUSA_OCOR"]  = $ocorrenciasAbonarFalta['DSOBSERVACAORECUSA'];
        } else {
            $apuracaoPonto["SOLICITOU_ABONAR_FALTA"]   = false;
            $apuracaoPonto["ID_SITUACAO_ABONARFALTA"]  = $ocorrenciasAbonarFalta['IDPROGOCORCONFIRMADA'];
            $apuracaoPonto["IDTIPODIACALENDARIO_OCOR"] = "";
        }

        if ($ocorrenciasCompensarFalta["COUNT_COMPENSACAO_FALTA"] > 0) {
            $apuracaoPonto["SOLICITOU_COMPENSAR_FALTA"]  = true;
            $apuracaoPonto["ID_SITUACAO_COMPENSARFALTA"] = $ocorrenciasCompensarFalta['IDPROGOCORCONFIRMADA'];
            $apuracaoPonto["NRPROGOCORRENCIA_OCOR"]      = $ocorrenciasCompensarFalta['NRPROGOCORRENCIA'];
            $apuracaoPonto["DSOBSERVACAO_OCOR"]          = $ocorrenciasAbonarFalta['DSOBSERVACAO'];
            $apuracaoPonto["DSOBSERVACAORECUSA_OCOR"]    = $ocorrenciasAbonarFalta['DSOBSERVACAORECUSA'];
        } else {
            $apuracaoPonto["SOLICITOU_COMPENSAR_FALTA"]  = false;
            $apuracaoPonto["ID_SITUACAO_COMPENSARFALTA"] = $ocorrenciasCompensarFalta['IDPROGOCORCONFIRMADA'];
        }

        // LEONARDOCARVALHO - 21/11/2017 - Calcula tempo de trabalho efetivo interno, total trabalhado subtraindo o tempo justificado
        $horastrabalhadasefetivas = floatval(str_replace(',', '.', $resumosSitufreq['TRABALH'])) - floatval(str_replace(',', '.', $resumosSitufreq['AU_JUST']));

        $apuracaoPonto["ENTRADA"]                  = $resumosSitufreq['ENTRADA'];      /*hora de entrada*/
        $apuracaoPonto["ALMOCO"]                   = $resumosSitufreq['ALMOCO'];       /*tempo de almoço*/
        $apuracaoPonto["SAIDA"]                    = $resumosSitufreq['SAIDA'];        /*hora de saída*/
        $apuracaoPonto["INJUSTIFICADAS"]           = $resumosSitufreq['AU_INJU'];      /*ausência intrajornada injustificada*/
        $apuracaoPonto["JUSTIFICADAS"]             = $resumosSitufreq['AU_JUST'];      /*ausência intrajornada justificada*/
        $apuracaoPonto["TOTAL_AUSENCIA"]           = $resumosSitufreq['AU_TOTA'];      /*ausência intrajornada tempo total*/
        $apuracaoPonto["HORASTRABALHADASEFETIVAS"]  = (String)$horastrabalhadasefetivas; /*horas trabalhadas normais/efetivas*/
        $apuracaoPonto["HORASTRABALHADAS"]         = $resumosSitufreq['TRABALH'];      /*horas trabalhadas com horas justificadas*/
        $apuracaoPonto["DIFERENCAJORNADA"]         = $resumosSitufreq['DIF_TRA'];      /*diferença sobre jornada*/
        $apuracaoPonto["SALDO_BH_ANTERIOR"]        = $resumosSitufreq['BH_DIA_ANT'];   /*banco de horas do dia anterior*/
        $apuracaoPonto["SALDO_DIA"]                = $resumosSitufreq['BH_DESC_DIA'];  /*banco de horas do dia*/
        $apuracaoPonto["BH_S_AC"]                  = $resumosSitufreq['BH_S_AC'];      /*banco de horas sem acréscimo*/
        $apuracaoPonto["SALDO_BH_ATUAL"]           = $resumosSitufreq['BH_C_AC'];      /*banco de horas com acréscimo*/
        $apuracaoPonto["FL_ATRA"]                  = $resumosSitufreq['FL_ATRA'];      /*faltas atrazos*/
        $apuracaoPonto["FL_DIAS"]                  = $resumosSitufreq['FL_DIAS'];      /*faltas dias*/
        $apuracaoPonto["HORASNORMAIS"]             = $resumosSitufreq['TRAB_NORM'];    /*horas base da jornada de trabalho*/

        return $apuracaoPonto;

    }

    public function getUltimoDiaApurado(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao
    ) {
        $sql = <<<SQL
            SELECT MAX(DTAPURACAOSITUFREQ) AS DTAPURACAOSITUFREQ
              FROM PTO_RESUMOSITUFREQ
             WHERE NRORG             = :P_NRORG
               AND NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
               AND NRVINCULOM        = :P_NRVINCULOM
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"             => $nrorg,
                "P_NRVINCULOM"        => $nrvinculom,
                "P_NRPERIODOAPURACAO" => $nrperiodoapuracao
            )
        );

        $resumoSitufreq = $query->fetchAll();

        if (!is_null($resumoSitufreq[0])) {
            return $resumoSitufreq[0];
        } else {
            return null;
        }
    }

    public function getSaldosPeriodoApuracao(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao,
        string $dtmescompetencia
    ) {
        /** @var $gpeVinculom Teknisa\Model\Gestaopessoas\Entities\GpeVinculom */
        $gpeVinculom = $this->entityManager->getRepository('Teknisa\Model\Gestaopessoas\Entities\GpeVinculom')
            ->findOneBy(array('nrvinculom' => $nrvinculom,
                'nrorg' => $nrorg));

        $nrtipovinculom = $gpeVinculom->getNrtipovinculom();

        $sql = <<<SQL
            SELECT
               NVL(SUM(RSF.QTREFAPURSITUFREQ), 0) AS QTREFAPURSITUFREQ,
               NVL(RSF.NRVINCULOM, 0) AS NRVINCULOM,
               NVL(RSF.NRSITUFREQUENCIA, 0) AS NRSITUFREQUENCIA,
               NVL(SFP.NREVENTOM, 0) AS NREVENTOM
              FROM PTO_RESUMOSITUFREQ RSF
              INNER JOIN
                (SELECT VCM.NRVINCULOM,
                  VCM.NRORG
                FROM GPE_TIPOVINCULOM TVM,
                  GPE_VINCULOM VCM
                INNER JOIN GPE_VINCULOH VCH
                ON (VCH.NRVINCULOM        = VCM.NRVINCULOM
                AND VCH.NRORG             = VCM.NRORG)
                WHERE VCH.DTMESCOMPETENC IN
                  (SELECT MAX(VIH.DTMESCOMPETENC)
                  FROM GPE_VINCULOH VIH
                  WHERE VIH.NRVINCULOM                          = VCM.NRVINCULOM
                  AND VIH.NRORG                                 = VCM.NRORG
                  AND VIH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')
                  )
                AND VCM.NRTIPOVINCULOM    = TVM.NRTIPOVINCULOM
                AND VCM.NRTIPOVINCULOM    = :P_NRTIPOVINCULOM
                AND VCM.NRORG             = :P_NRORG
                AND VCM.DTRESCISAOVINC IS NULL
                AND (TVM.NRORG            = VCM.NRORG
                OR TVM.NRORG              = 0)
                ) VIN ON VIN.NRVINCULOM   = RSF.NRVINCULOM
              INNER JOIN PTO_SITUFREQPARAM SFP
                 ON SFP.NRSITUFREQUENCIA = RSF.NRSITUFREQUENCIA
              AND VIN.NRORG               = RSF.NRORG
              WHERE RSF.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
              AND RSF.IDCONFIRMADA        = 'S'
              --AND RSF.NRLOTELANCEVENTO IS NULL
              AND SFP.NREVENTOM IS NOT NULL
              AND SFP.NREVENTOM != 1040
              AND RSF.NRVINCULOM = :P_NRVINCULOM
              GROUP BY RSF.NRVINCULOM,
               RSF.NRSITUFREQUENCIA,
               SFP.NREVENTOM

              UNION ALL

              SELECT
               NVL(COUNT(RSF.QTREFAPURSITUFREQ), 0) AS QTREFAPURSITUFREQ,
               NVL(RSF.NRVINCULOM, 0) AS NRVINCULOM,
               NVL(RSF.NRSITUFREQUENCIA, 0) AS NRSITUFREQUENCIA,
               NVL(SFP.NREVENTOM, 0) AS NREVENTOM
              FROM PTO_RESUMOSITUFREQ RSF
              INNER JOIN
                (SELECT VCM.NRVINCULOM,
                  VCM.NRORG
                FROM GPE_TIPOVINCULOM TVM,
                  GPE_VINCULOM VCM
                INNER JOIN GPE_VINCULOH VCH
                ON (VCH.NRVINCULOM        = VCM.NRVINCULOM
                AND VCH.NRORG             = VCM.NRORG)
                WHERE VCH.DTMESCOMPETENC IN
                  (SELECT MAX(VIH.DTMESCOMPETENC)
                  FROM GPE_VINCULOH VIH
                  WHERE VIH.NRVINCULOM                          = VCM.NRVINCULOM
                  AND VIH.NRORG                                 = VCM.NRORG
                  AND VIH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')
                  )
                AND VCM.NRTIPOVINCULOM    = TVM.NRTIPOVINCULOM
                AND VCM.NRTIPOVINCULOM    = :P_NRTIPOVINCULOM
                AND VCM.NRORG             = :P_NRORG
                AND VCM.DTRESCISAOVINC IS NULL
                AND (TVM.NRORG            = VCM.NRORG
                OR TVM.NRORG              = 0)
                ) VIN ON VIN.NRVINCULOM   = RSF.NRVINCULOM
              INNER JOIN PTO_SITUFREQPARAM SFP
                 ON SFP.NRSITUFREQUENCIA = RSF.NRSITUFREQUENCIA
              AND VIN.NRORG               = RSF.NRORG
              WHERE RSF.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
              AND RSF.IDCONFIRMADA        = 'S'
              --AND RSF.NRLOTELANCEVENTO IS NULL
              AND SFP.NREVENTOM IS NOT NULL
              AND SFP.NREVENTOM = 1040
              AND RSF.NRVINCULOM = :P_NRVINCULOM
              GROUP BY RSF.NRVINCULOM,
               RSF.NRSITUFREQUENCIA,
               SFP.NREVENTOM
              ORDER BY NRVINCULOM
SQL;

        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                "P_NRTIPOVINCULOM"    => $nrtipovinculom,
                "P_DTMESCOMPETENCIA"  => $dtmescompetencia,
                "P_NRORG"             => $nrorg,
                "P_NRPERIODOAPURACAO" => $nrperiodoapuracao,
                "P_NRVINCULOM"        => $nrvinculom
            )
        );
        $resumoSitufreq = $query->fetchAll();

        if (isset($resumoSitufreq[0])) {
            $saldosPeriodoApuracao = $this->factorySaldoPeriodoApuracao($resumoSitufreq);
            return $saldosPeriodoApuracao;
        } else {
            $saldosPeriodoApuracao = array();

            $saldosPeriodoApuracao["HRFALTA"]   = 0;
            $saldosPeriodoApuracao["HREXTRAS"]  = 0;
            $saldosPeriodoApuracao["DIASFALTA"] = 0;
            $saldosPeriodoApuracao["HRADICIONALNOTURNO"] = 0;

            return $saldosPeriodoApuracao;
        }
    }

    private function factorySaldoPeriodoApuracao($resumoSitufreq) {
        $saldosPeriodoApuracao = array();

        $saldosPeriodoApuracao["HRFALTA"]   = 0;
        $saldosPeriodoApuracao["HREXTRAS"]  = 0;
        $saldosPeriodoApuracao["DIASFALTA"] = 0;
        $saldosPeriodoApuracao["HRADICIONALNOTURNO"] = 0;

        foreach ($resumoSitufreq as $key => $saldos) {
            if ($saldos["NRSITUFREQUENCIA"] == '10100') // ATRASOS
                $saldosPeriodoApuracao["HRFALTA"]   = $saldos["QTREFAPURSITUFREQ"];
            else if ($saldos["NRSITUFREQUENCIA"] == '2300' || $saldos["NRSITUFREQUENCIA"] == '8000') // HORAS EXTRAS
                $saldosPeriodoApuracao["HREXTRAS"]  = $saldos["QTREFAPURSITUFREQ"];
            else if ($saldos["NRSITUFREQUENCIA"] == '10500') // FALTAS
                $saldosPeriodoApuracao["DIASFALTA"] = $saldos["QTREFAPURSITUFREQ"];
            else if ($saldos["NRSITUFREQUENCIA"] == '9000') // ADICIONAL_NOTURNO
                $saldosPeriodoApuracao["HRADICIONALNOTURNO"] = $saldos["QTREFAPURSITUFREQ"];
        }

        return $saldosPeriodoApuracao;
    }


    /**
     * Verifica se existem ocorrencias pendentes para o vinculo no dia a ser apurado
     * @return null
     */
    public function verificaExisteOcorrenciaPendente(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
                COUNT(*) AS COUNT_OCORRENCIAS_PENDENTES
              FROM PTO_PROGOCORRENCIA PROG
             WHERE PROG.NRORG                = :P_NRORG
               AND IDATIVO                   = 'S'
               AND PROG.NRVINCULOM           = :P_NRVINCULOM
               AND PROG.DTINICIOPROGOCOR     = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
               AND PROG.IDPROGOCORCONFIRMADA = 'P'
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrenciasPendentes = $query->fetchAll();
        if (isset($ocorrenciasPendentes[0])) {
            return $ocorrenciasPendentes[0];
        } else {
            return null;
        }
    }

    /**
     * Verifica se existem resumos de situação de frequência de faltas diurnas ou noturnas para o dia
     * @return null
     */
    public function verificaExisteResumoFaltaDiurnaNoturna(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
              COUNT(*) AS COUNT_FALTAS
             FROM PTO_RESUMOSITUFREQ RES
            WHERE NRORG              = :P_NRORG
              AND NRVINCULOM         = :P_NRVINCULOM
              AND NRPERIODOAPURACAO  = :P_NRPERIODOAPURACAO
              AND DTAPURACAOSITUFREQ = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND NRSITUFREQUENCIA  IN (10500 /*Falta Diurna*/, 10600 /*Falta Noturna*/)
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"             => $nrorg,
                "P_NRVINCULOM"        => $nrvinculom,
                "P_NRPERIODOAPURACAO" => $nrperiodoapuracao,
                "P_DTAPURACAOPONTO"   => $dtapuracaoponto
            )
        );

        $faltasEncontradas = $query->fetchAll();
        if (isset($faltasEncontradas[0])) {
            return $faltasEncontradas[0];
        } else {
            return null;
        }
    }

    /**
     * Verifica se existe ocorrências de feriado no calendário para a data
     * @return null
     */
    public function verificaOcorrenciasCalendario(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto,
        string $dtmescompetencia
    ) {
        $sql = <<<SQL
            SELECT
              COUNT(*) AS COUNT_OCORRENCIAS_FERIADO
              /*OCORCALENDARIO.DTCALENDARIO,
              OCORCALENDARIO.DSOBSCALEND,
              TPCALENDARIO.DSTPCALENDARIO,
              TPCALENDARIO.DSABREVTPCALEND,
              TPCALENDARIO.IDTIPODIA*/
             FROM OCORCALENDARIO
             JOIN TPCALENDARIO
               ON TPCALENDARIO.NRTPCALENDARIO = OCORCALENDARIO.NRTPCALENDARIO
            WHERE OCORCALENDARIO.NRORG = :P_NRORG
              AND OCORCALENDARIO.DTCALENDARIO = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND OCORCALENDARIO.NRCALENDARIO IN (SELECT NRCALENDARIO
                                    FROM ESTRUTURAH
                                   WHERE NRESTRUTURAM = (
                                        SELECT MOV.NRESTRUTURAM
                                        FROM GPE_MOVIMENTACAO MOV
                                        JOIN GPE_PARAMORG PO ON PO.NRORG = MOV.NRORG
                                        WHERE MOV.NRORG = :P_NRORG
                                            AND MOV.NRVINCULOM = :P_NRVINCULOM
                                            AND MOV.DTINIMOVIMENT <= LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY'))
                                            AND (MOV.DTFIMMOVIMENT IS NULL OR MOV.DTFIMMOVIMENT >  LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')))
                                            AND MOV.NRTIPOESTRUTURA IN (PO.NRTPESTRUTLEGAL, PO.NRTPESTRUTCALEND)
                                        ORDER BY DECODE(MOV.NRTIPOESTRUTURA, PO.NRTPESTRUTCALEND, 1, PO.NRTPESTRUTLEGAL, 2, 3), MOV.DTINIMOVIMENT DESC
                                        FETCH FIRST ROW ONLY
                                   )
                                    AND DTMESCOMPETENC IN (
                                        SELECT MAX(DTMESCOMPETENC)
                                           FROM ESTRUTURAH H1
                                          WHERE H1.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
                                            AND H1.NRORG = ESTRUTURAH.NRORG
                                            AND H1.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')
                                        )
                                    AND NRORG = :P_NRORG)
              AND OCORCALENDARIO.NRORG = :P_NRORG
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"            => $nrorg,
                "P_NRVINCULOM"       => $nrvinculom,
                "P_DTAPURACAOPONTO"  => $dtapuracaoponto,
                "P_DTMESCOMPETENCIA" => $dtmescompetencia
            )
        );

        $ocorrenciasCalendario = $query->fetchAll();
        if (isset($ocorrenciasCalendario[0])) {
            return $ocorrenciasCalendario[0];
        } else {
            return null;
        }
    }

}